@@ -1,10 +1,18 @@
 from __future__ import annotations
 
+import sys
 from typing import ClassVar, Dict, Iterable, List, Optional, Type, Union
 
 from markdown_it import MarkdownIt
 from markdown_it.token import Token
 
+if sys.version_info >= (3, 8):
+    from typing import get_args
+else:
+    from typing_extensions import get_args  # pragma: no cover
+
+from rich.table import Table
+
 from . import box
 from ._loop import loop_first
 from ._stack import Stack
@@ -223,6 +231,117 @@ class HorizontalRule(MarkdownElement):
         yield Rule(style=style)
 
 
+class TableElement(MarkdownElement):
+    """MarkdownElement corresponding to `table_open`."""
+
+    def __init__(self) -> None:
+        self.header: TableHeaderElement | None = None
+        self.body: TableBodyElement | None = None
+
+    def on_child_close(
+        self, context: "MarkdownContext", child: "MarkdownElement"
+    ) -> bool:
+        if isinstance(child, TableHeaderElement):
+            self.header = child
+        elif isinstance(child, TableBodyElement):
+            self.body = child
+        else:
+            raise RuntimeError("Couldn't process markdown table.")
+        return False
+
+    def __rich_console__(
+        self, console: Console, options: ConsoleOptions
+    ) -> RenderResult:
+        table = Table(box=box.SIMPLE_HEAVY)
+
+        assert self.header is not None
+        assert self.header.row is not None
+        for column in self.header.row.cells:
+            table.add_column(column.content)
+
+        assert self.body is not None
+        for row in self.body.rows:
+            row_content = [element.content for element in row.cells]
+            table.add_row(*row_content)
+
+        yield table
+
+
+class TableHeaderElement(MarkdownElement):
+    """MarkdownElement corresponding to `thead_open` and `thead_close`."""
+
+    def __init__(self) -> None:
+        self.row: TableRowElement | None = None
+
+    def on_child_close(
+        self, context: "MarkdownContext", child: "MarkdownElement"
+    ) -> bool:
+        assert isinstance(child, TableRowElement)
+        self.row = child
+        return False
+
+
+class TableBodyElement(MarkdownElement):
+    """MarkdownElement corresponding to `tbody_open` and `tbody_close`."""
+
+    def __init__(self) -> None:
+        self.rows: list[TableRowElement] = []
+
+    def on_child_close(
+        self, context: "MarkdownContext", child: "MarkdownElement"
+    ) -> bool:
+        assert isinstance(child, TableRowElement)
+        self.rows.append(child)
+        return False
+
+
+class TableRowElement(MarkdownElement):
+    """MarkdownElement corresponding to `tr_open` and `tr_close`."""
+
+    def __init__(self) -> None:
+        self.cells: List[TableDataElement] = []
+
+    def on_child_close(
+        self, context: "MarkdownContext", child: "MarkdownElement"
+    ) -> bool:
+        assert isinstance(child, TableDataElement)
+        self.cells.append(child)
+        return False
+
+
+class TableDataElement(MarkdownElement):
+    """MarkdownElement corresponding to `td_open` and `td_close`
+    and `th_open` and `th_close`."""
+
+    @classmethod
+    def create(cls, markdown: "Markdown", token: Token) -> "MarkdownElement":
+        style = str(token.attrs.get("style" "")) or ""
+
+        justify: JustifyMethod
+        if "text-align:right" in style:
+            justify = "right"
+        elif "text-align:center" in style:
+            justify = "center"
+        elif "text-align:left" in style:
+            justify = "left"
+        else:
+            justify = "default"
+
+        assert justify in get_args(JustifyMethod)
+        return cls(justify=justify)
+
+    def __init__(self, justify: JustifyMethod) -> None:
+        self.content: TextType = ""
+        self.justify = justify
+
+    def on_text(self, context: "MarkdownContext", text: TextType) -> None:
+        plain = text.plain if isinstance(text, Text) else text
+        style = text.style if isinstance(text, Text) else ""
+        self.content = Text(
+            plain, justify=self.justify, style=context.style_stack.current
+        )
+
+
 class ListElement(MarkdownElement):
     """A list element."""
 
@@ -426,6 +545,12 @@ class Markdown(JupyterMixin):
         "ordered_list_open": ListElement,
         "list_item_open": ListItem,
         "image": ImageItem,
+        "table_open": TableElement,
+        "tbody_open": TableBodyElement,
+        "thead_open": TableHeaderElement,
+        "tr_open": TableRowElement,
+        "td_open": TableDataElement,
+        "th_open": TableDataElement,
     }
 
     inlines = {"em", "strong", "code", "s"}
@@ -440,7 +565,7 @@ class Markdown(JupyterMixin):
         inline_code_lexer: Optional[str] = None,
         inline_code_theme: Optional[str] = None,
     ) -> None:
-        parser = MarkdownIt().enable("strikethrough")
+        parser = MarkdownIt().enable("strikethrough").enable("table")
         self.markup = markup
         self.parsed = parser.parse(markup)
         self.code_theme = code_theme
@@ -557,6 +682,7 @@ class Markdown(JupyterMixin):
                     if should_render:
                         if new_line:
                             yield _new_line_segment
+
                         yield from console.render(element, context.options)
                 elif self_closing:  # SELF-CLOSING tags (e.g. text, code, image)
                     context.stack.pop()
@@ -580,7 +706,6 @@ class Markdown(JupyterMixin):
 
 
 if __name__ == "__main__":  # pragma: no cover
-
     import argparse
     import sys
 
@@ -652,6 +777,7 @@ if __name__ == "__main__":  # pragma: no cover
     else:
         with open(args.path, "rt", encoding="utf-8") as markdown_file:
             markdown_body = markdown_file.read()
+
     markdown = Markdown(
         markdown_body,
         justify="full" if args.justify else "left",
